@@ -36,7 +36,6 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <gio/gdesktopappinfo.h>
 #include <gio/gunixfdlist.h>
@@ -1838,7 +1837,7 @@ update_screen_cb (GObject             *source_object,
                   gpointer             user_data)
 {
         GError *error = NULL;
-        guint percentage;
+        int percentage;
         GVariant *new_percentage;
         GsdMediaKeysManager *manager = GSD_MEDIA_KEYS_MANAGER (user_data);
 
@@ -1852,8 +1851,9 @@ update_screen_cb (GObject             *source_object,
         }
 
         /* update the dialog with the new value */
-        g_variant_get (new_percentage, "(u)", &percentage);
-        show_osd (manager, "display-brightness-symbolic", NULL, percentage);
+        g_variant_get (new_percentage, "(i)", &percentage);
+        if (percentage >= 0)
+                show_osd (manager, "display-brightness-symbolic", NULL, percentage);
         g_variant_unref (new_percentage);
 }
 
@@ -2127,6 +2127,7 @@ static void
 on_accelerator_activated (ShellKeyGrabber     *grabber,
                           guint                accel_id,
                           guint                deviceid,
+                          guint                timestamp,
                           GsdMediaKeysManager *manager)
 {
         guint i;
@@ -2140,9 +2141,9 @@ on_accelerator_activated (ShellKeyGrabber     *grabber,
                         continue;
 
                 if (key->key_type == CUSTOM_KEY)
-                        do_custom_action (manager, deviceid, key, GDK_CURRENT_TIME);
+                        do_custom_action (manager, deviceid, key, timestamp);
                 else
-                        do_action (manager, deviceid, key->key_type, GDK_CURRENT_TIME);
+                        do_action (manager, deviceid, key->key_type, timestamp);
                 return;
         }
 }
