@@ -30,9 +30,8 @@
 
 #include <locale.h>
 
-#include <glib.h>
 #include <glib/gi18n.h>
-#include <gdk/gdk.h>
+#include <gio/gio.h>
 
 #include "gnome-settings-session.h"
 #include "gnome-settings-profile.h"
