@@ -487,7 +487,7 @@ apply_configuration_from_filename (GsdXrandrManager *manager,
  * We just return whether setting the configuration succeeded.
  */
 static gboolean
-apply_configuration (GsdXrandrManager *manager, GnomeRRConfig *config, guint32 timestamp, gboolean show_error, gboolean save_configuration)
+apply_configuration (GsdXrandrManager *manager, GnomeRRConfig *config, guint32 timestamp, gboolean save_configuration)
 {
         GsdXrandrManagerPrivate *priv = manager->priv;
         GError *error;
@@ -505,8 +505,6 @@ apply_configuration (GsdXrandrManager *manager, GnomeRRConfig *config, guint32 t
         } else {
                 log_msg ("Could not switch to the following configuration (timestamp %u): %s\n", timestamp, error->message);
                 log_configuration (config);
-                if (show_error)
-                        error_message (manager, _("Could not switch the monitor configuration"), error, NULL);
                 g_error_free (error);
         }
 
@@ -1048,16 +1046,16 @@ turn_on_and_get_rightmost_offset (GnomeRRScreen *screen, GnomeRROutputInfo *info
         return x;
 }
 
-/* Used from qsort(); compares outputs based on their X position */
+/* Used from g_ptr_array_sort(); compares outputs based on their X position */
 static int
-compare_output_positions (const void *a, const void *b)
+compare_output_positions (gconstpointer a, gconstpointer b)
 {
-        GnomeRROutputInfo *oa = (GnomeRROutputInfo *) a;
-        GnomeRROutputInfo *ob = (GnomeRROutputInfo *) b;
+        GnomeRROutputInfo **oa = (GnomeRROutputInfo **) a;
+        GnomeRROutputInfo **ob = (GnomeRROutputInfo **) b;
         int xa, xb;
 
-        gnome_rr_output_info_get_geometry (oa, &xa, NULL, NULL, NULL);
-        gnome_rr_output_info_get_geometry (ob, &xb, NULL, NULL, NULL);
+        gnome_rr_output_info_get_geometry (*oa, &xa, NULL, NULL, NULL);
+        gnome_rr_output_info_get_geometry (*ob, &xb, NULL, NULL, NULL);
 
         return xb - xa;
 }
@@ -1071,39 +1069,30 @@ static gboolean
 trim_rightmost_outputs_that_dont_fit_in_framebuffer (GnomeRRScreen *rr_screen, GnomeRRConfig *config)
 {
         GnomeRROutputInfo **outputs;
-        GnomeRROutputInfo **sorted_outputs;
-        int num_on_outputs;
-        int i, j;
+        int i;
         gboolean applicable;
+        GPtrArray *sorted_outputs;
 
         outputs = gnome_rr_config_get_outputs (config);
+        g_return_val_if_fail (outputs != NULL, FALSE);
 
         /* How many are on? */
 
-        num_on_outputs = 0;
+        sorted_outputs = g_ptr_array_new ();
         for (i = 0; outputs[i] != NULL; i++) {
                 if (gnome_rr_output_info_is_active (outputs[i]))
-                        num_on_outputs++;
+                        g_ptr_array_add (sorted_outputs, outputs[i]);
         }
 
         /* Lay them out from left to right */
 
-        sorted_outputs = g_new (GnomeRROutputInfo *, num_on_outputs);
-        j = 0;
-        for (i = 0; outputs[i] != NULL; i++) {
-                if (gnome_rr_output_info_is_active (outputs[i])) {
-                        sorted_outputs[j] = outputs[i];
-                        j++;
-                }
-        }
-
-        qsort (sorted_outputs, num_on_outputs, sizeof (sorted_outputs[0]), compare_output_positions);
+        g_ptr_array_sort (sorted_outputs, compare_output_positions);
 
         /* Trim! */
 
         applicable = FALSE;
 
-        for (i = num_on_outputs - 1; i >= 0; i--) {
+        for (i = sorted_outputs->len - 1; i >= 0; i--) {
                 GError *error = NULL;
                 gboolean is_bounds_error;
 
@@ -1117,13 +1106,13 @@ trim_rightmost_outputs_that_dont_fit_in_framebuffer (GnomeRRScreen *rr_screen, G
                 if (!is_bounds_error)
                         break;
 
-                gnome_rr_output_info_set_active (sorted_outputs[i], FALSE);
+                gnome_rr_output_info_set_active (sorted_outputs->pdata[i], FALSE);
         }
 
         if (config_is_all_off (config))
                 applicable = FALSE;
 
-        g_free (sorted_outputs);
+        g_ptr_array_free (sorted_outputs, FALSE);
 
         return applicable;
 }
@@ -1341,15 +1330,10 @@ generate_fn_f7_configs (GsdXrandrManager *mgr)
 static void
 error_message (GsdXrandrManager *mgr, const char *primary_text, GError *error_to_display, const char *secondary_text)
 {
-        GtkWidget *dialog;
-
-        dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                         "%s", primary_text);
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s",
-                                                  error_to_display ? error_to_display->message : secondary_text);
-
-        gtk_dialog_run (GTK_DIALOG (dialog));
-        gtk_widget_destroy (dialog);
+    g_warning("%s\n%s\n%s",
+              primary_text? primary_text : "",
+              secondary_text? secondary_text : "",
+              error_to_display? error_to_display->message : "");
 }
 
 static void
@@ -1385,7 +1369,6 @@ handle_fn_f7 (GsdXrandrManager *mgr, guint32 timestamp)
                 g_error_free (error);
 
                 log_msg ("%s\n", str);
-                error_message (mgr, str, NULL, _("Trying to switch the monitor configuration anyway."));
                 g_free (str);
         }
 
@@ -1441,7 +1424,7 @@ handle_fn_f7 (GsdXrandrManager *mgr, guint32 timestamp)
                 if (timestamp < server_timestamp)
                         timestamp = server_timestamp;
 
-                success = apply_configuration (mgr, priv->fn_f7_configs[mgr->priv->current_fn_f7_config], timestamp, TRUE, TRUE);
+                success = apply_configuration (mgr, priv->fn_f7_configs[mgr->priv->current_fn_f7_config], timestamp, TRUE);
 
                 if (success) {
                         log_msg ("Successfully switched to configuration (timestamp %u):\n", timestamp);
@@ -1502,15 +1485,13 @@ get_next_rotation (GnomeRRRotation allowed_rotations, GnomeRRRotation current_ro
 
 struct {
         GnomeRRRotation rotation;
-        /* evdev */
-        gboolean x_axis_inversion;
-        gboolean y_axis_inversion;
-        gboolean axes_swap;
+        /* Coordinate Transformation Matrix */
+        gfloat matrix[9];
 } evdev_rotations[] = {
-        { GNOME_RR_ROTATION_0, 0, 0, 0 },
-        { GNOME_RR_ROTATION_90, 1, 0, 1 },
-        { GNOME_RR_ROTATION_180, 1, 1, 0 },
-        { GNOME_RR_ROTATION_270, 0, 1, 1 }
+        { GNOME_RR_ROTATION_0, {1, 0, 0, 0, 1, 0, 0, 0, 1}},
+        { GNOME_RR_ROTATION_90, {0, -1, 1, 1, 0, 0, 0, 0, 1}},
+        { GNOME_RR_ROTATION_180, {-1, 0, 1, 0, -1, 1, 0, 0, 1}},
+        { GNOME_RR_ROTATION_270, {0, 1, 0, -1, 0, 1, 0,  0, 1}}
 };
 
 static guint
@@ -1566,6 +1547,7 @@ rotate_touchscreens (GsdXrandrManager *mgr,
         XDeviceInfo *device_info;
         gint n_devices;
         guint i, rot_idx;
+        Atom float_atom;
 
         if (!supports_xinput_devices ())
                 return;
@@ -1578,6 +1560,8 @@ rotate_touchscreens (GsdXrandrManager *mgr,
 
         rot_idx = get_rotation_index (rotation);
 
+        float_atom = XInternAtom(GDK_DISPLAY_XDISPLAY(gdk_display_get_default()), "FLOAT", True);
+
         for (i = 0; i < n_devices; i++) {
                 if (is_wacom_tablet_device  (mgr, &device_info[i])) {
                         g_debug ("Not rotating tablet device '%s'", device_info[i].name);
@@ -1587,13 +1571,13 @@ rotate_touchscreens (GsdXrandrManager *mgr,
                 if (device_info_is_touchscreen (&device_info[i]) ||
                             device_info_is_tablet (&device_info[i])) {
                         XDevice *device;
-                        char c = evdev_rotations[rot_idx].axes_swap;
-                        PropertyHelper axes_swap = {
-                                .name = "Evdev Axes Swap",
-                                .nitems = 1,
-                                .format = 8,
-                                .type   = XA_INTEGER,
-                                .data.c = &c,
+                        gfloat *m = evdev_rotations[rot_idx].matrix;
+                        PropertyHelper matrix = {
+                                .name = "Coordinate Transformation Matrix",
+                                .nitems = 9,
+                                .format = 32,
+                                .type = float_atom,
+                                .data.i = (int *)m,
                         };
 
                         g_debug ("About to rotate '%s'", device_info[i].name);
@@ -1603,26 +1587,18 @@ rotate_touchscreens (GsdXrandrManager *mgr,
                         if (gdk_error_trap_pop () || (device == NULL))
                                 continue;
 
-                        if (device_set_property (device, device_info[i].name, &axes_swap) != FALSE) {
-                                char axis[] = {
-                                        evdev_rotations[rot_idx].x_axis_inversion,
-                                        evdev_rotations[rot_idx].y_axis_inversion
-                                };
-                                PropertyHelper axis_invert = {
-                                        .name = "Evdev Axis Inversion",
-                                        .nitems = 2,
-                                        .format = 8,
-                                        .type   = XA_INTEGER,
-                                        .data.c = axis,
-                                };
-
-                                device_set_property (device, device_info[i].name, &axis_invert);
-
-                                g_debug ("Rotated '%s' to configuration '%d, %d, %d'",
+                        if (device_set_property (device, device_info[i].name, &matrix) != FALSE) {
+                                g_debug ("Rotated '%s' to configuration '%f, %f, %f, %f, %f, %f, %f, %f, %f'\n",
                                          device_info[i].name,
-                                         evdev_rotations[rot_idx].x_axis_inversion,
-                                         evdev_rotations[rot_idx].y_axis_inversion,
-                                         evdev_rotations[rot_idx].axes_swap);
+                                         evdev_rotations[rot_idx].matrix[0],
+                                         evdev_rotations[rot_idx].matrix[1],
+                                         evdev_rotations[rot_idx].matrix[2],
+                                         evdev_rotations[rot_idx].matrix[3],
+                                         evdev_rotations[rot_idx].matrix[4],
+                                         evdev_rotations[rot_idx].matrix[5],
+                                         evdev_rotations[rot_idx].matrix[6],
+                                         evdev_rotations[rot_idx].matrix[7],
+                                         evdev_rotations[rot_idx].matrix[8]);
                         }
 
                         XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), device);
@@ -1648,7 +1624,7 @@ handle_rotate_windows (GsdXrandrManager *mgr,
         int num_allowed_rotations;
         GnomeRRRotation allowed_rotations;
         GnomeRRRotation next_rotation;
-        gboolean success, show_error;
+        gboolean success;
 
         g_debug ("Handling XF86RotateWindows with rotation %d", rotation);
 
@@ -1672,17 +1648,15 @@ handle_rotate_windows (GsdXrandrManager *mgr,
                         g_debug ("No rotations are supported other than the current one; XF86RotateWindows key will do nothing");
                         goto out;
                 }
-                show_error = TRUE;
         } else {
                 next_rotation = rotation;
-                show_error = FALSE;
         }
 
         /* Rotate */
 
         gnome_rr_output_info_set_rotation (rotatable_output_info, next_rotation);
 
-        success = apply_configuration (mgr, current, timestamp, show_error, TRUE);
+        success = apply_configuration (mgr, current, timestamp, TRUE);
         if (success)
                 rotate_touchscreens (mgr, next_rotation);
 
@@ -1731,7 +1705,7 @@ auto_configure_outputs (GsdXrandrManager *manager, guint32 timestamp)
         g_debug ("xrandr auto-configure");
         config = make_default_setup (manager);
         if (config) {
-                apply_configuration (manager, config, timestamp, TRUE, FALSE);
+                apply_configuration (manager, config, timestamp, TRUE);
                 g_object_unref (config);
         } else {
                 g_debug ("No applicable configuration found during auto-configure");
@@ -1914,7 +1888,7 @@ apply_default_boot_configuration (GsdXrandrManager *mgr, guint32 timestamp)
                 /* We don't save the configuration (the "false" parameter to the following function) because we don't want to
                  * install a user-side setting when *here* we are using a system-default setting.
                  */
-                apply_configuration (mgr, config, timestamp, TRUE, FALSE);
+                apply_configuration (mgr, config, timestamp, TRUE);
                 g_object_unref (config);
         }
 }
@@ -2014,7 +1988,7 @@ turn_off_laptop_display (GsdXrandrManager *manager, guint32 timestamp)
                  * wouldn't want to restore a configuration with the laptop's display turned off, if at some
                  * point later the user booted his laptop with the lid open.
                  */
-                apply_configuration (manager, config, timestamp, FALSE, FALSE);
+                apply_configuration (manager, config, timestamp, FALSE);
         }
 
         g_object_unref (config);
@@ -2260,15 +2234,10 @@ on_bus_gotten (GObject             *source_object,
         GDBusInterfaceInfo **infos;
         int i;
 
-        if (manager->priv->bus_cancellable == NULL ||
-            g_cancellable_is_cancelled (manager->priv->bus_cancellable)) {
-                g_warning ("Operation has been cancelled, so not retrieving session bus");
-                return;
-        }
-
         connection = g_bus_get_finish (res, &error);
         if (connection == NULL) {
-                g_warning ("Could not get session bus: %s", error->message);
+                if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+                        g_warning ("Could not get session bus: %s", error->message);
                 g_error_free (error);
                 return;
         }
