@@ -14,6 +14,8 @@ import shutil
 import sys
 from glob import glob
 
+from gi.repository import GLib
+
 try:
     import dbusmock
 except ImportError:
@@ -34,14 +36,6 @@ if subprocess.call(['which', 'gnome-session'], stdout=subprocess.PIPE) != 0:
 top_builddir = os.environ.get('TOP_BUILDDIR',
                               os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 
-# from gnome-session/gsm-inhibitor.h
-GSM_INHIBITOR_FLAG_LOGOUT      = 1 << 0
-GSM_INHIBITOR_FLAG_SWITCH_USER = 1 << 1
-GSM_INHIBITOR_FLAG_SUSPEND     = 1 << 2
-GSM_INHIBITOR_FLAG_IDLE        = 1 << 3
-GSM_INHIBITOR_FLAG_AUTOMOUNT   = 1 << 4
-
-
 def set_nonblock(fd):
     '''Set a file object to non-blocking'''
 
@@ -64,6 +58,11 @@ class GSDTestCase(dbusmock.DBusTestCase):
         os.environ['LC_MESSAGES'] = 'C'
         klass.workdir = tempfile.mkdtemp(prefix='gsd-power-test')
 
+        # start X.org server with dummy driver; this is needed until Xvfb
+        # supports XRandR:
+        # http://lists.x.org/archives/xorg-devel/2013-January/035114.html
+        klass.start_xorg()
+
         # tell dconf and friends to use our config/runtime directories
         os.environ['XDG_CONFIG_HOME'] = os.path.join(klass.workdir, 'config')
         os.environ['XDG_DATA_HOME'] = os.path.join(klass.workdir, 'data')
@@ -83,13 +82,17 @@ class GSDTestCase(dbusmock.DBusTestCase):
         set_nonblock(klass.p_notify.stdout)
 
         klass.start_session()
+        klass.start_monitor()
 
         klass.settings_session = Gio.Settings('org.gnome.desktop.session')
 
     @classmethod
     def tearDownClass(klass):
-        klass.stop_session()
+        klass.p_notify.terminate()
+        klass.p_notify.wait()
+        klass.stop_monitor()
         dbusmock.DBusTestCase.tearDownClass()
+        klass.stop_xorg()
         shutil.rmtree(klass.workdir)
 
     def run(self, result=None):
@@ -126,39 +129,27 @@ class GSDTestCase(dbusmock.DBusTestCase):
             os.makedirs(d)
         shutil.copy(os.path.join(os.path.dirname(__file__), 'dummyapp.desktop'), d)
 
-        klass.session_log = open(os.path.join(klass.workdir, 'gnome-session.log'), 'wb')
-        klass.session = subprocess.Popen(['gnome-session', '-f',
-                                          '-a', os.path.join(klass.workdir, 'autostart'),
-                                          '--session=dummy', '--debug'],
-                                         stdout=klass.session_log,
+    @classmethod
+    def start_monitor(klass):
+        '''Start dbus-monitor'''
+
+        # You can rename the log file to *.log if you want to see it on test
+        # case failures
+        klass.monitor_log = open(os.path.join(klass.workdir, 'dbus-monitor.out'), 'wb')
+        klass.monitor = subprocess.Popen(['dbus-monitor', '--monitor'],
+                                         stdout=klass.monitor_log,
                                          stderr=subprocess.STDOUT)
 
-        # wait until the daemon is on the bus
-        try:
-            klass.wait_for_bus_object('org.gnome.SessionManager',
-                                      '/org/gnome/SessionManager')
-        except:
-            # on failure, print log
-            with open(klass.session_log.name) as f:
-                print('----- session log -----\n%s\n------' % f.read())
-            raise
-
-        klass.obj_session_mgr = klass.session_bus_con.get_object(
-            'org.gnome.SessionManager', '/org/gnome/SessionManager')
-
-        # give g-session some time to lazily initialize idle timers, etc.
-        time.sleep(3)
-
     @classmethod
-    def stop_session(klass):
-        '''Stop GNOME session'''
+    def stop_monitor(klass):
+        '''Stop dbus-monitor'''
 
-        assert klass.session
-        klass.session.terminate()
-        klass.session.wait()
+        assert klass.monitor
+        klass.monitor.terminate()
+        klass.monitor.wait()
 
-        klass.session_log.flush()
-        klass.session_log.close()
+        klass.monitor_log.flush()
+        klass.monitor_log.close()
 
     def start_logind(self):
         '''start mock logind'''
@@ -188,6 +179,58 @@ class GSDTestCase(dbusmock.DBusTestCase):
         self.logind.terminate()
         self.logind.wait()
 
+    @classmethod
+    def start_xorg(klass):
+        '''start X.org server with dummy driver'''
+
+        conf = os.path.join(os.path.dirname(__file__), 'xorg-dummy.conf')
+
+        # some distros like Fedora install Xorg as suid root; copy it into our
+        # workdir to drop the suid bit and run it as user
+        out = GLib.find_program_in_path ('Xorg')
+        if not out:
+            sys.stderr.write('ERROR: Xorg not installed\n')
+            sys.exit(1)
+        xorg = os.path.join(klass.workdir, 'Xorg')
+        shutil.copy(out, xorg)
+
+        display_num = 99
+
+        if os.path.isfile('/tmp/.X%d-lock' % display_num):
+            sys.stderr.write('Cannot start X.org, an instance already exists\n')
+            sys.exit(1)
+
+        # You can rename the log file to *.log if you want to see it on test
+        # case failures
+        log = os.path.join(klass.workdir, 'Xorg.out')
+        klass.xorg = subprocess.Popen([xorg, '-config', conf, '-logfile', log, ':%d' % display_num],
+                                      stderr=subprocess.PIPE)
+        os.environ['DISPLAY'] = ':%d' % display_num
+
+        # wait until the server is ready
+        timeout = 50
+        while timeout > 0:
+            time.sleep(0.1)
+            timeout -= 1
+            if klass.xorg.poll():
+                # ended prematurely
+                timeout = -1
+                break
+            if subprocess.call(['xprop', '-root'], stdout=subprocess.PIPE,
+                               stderr=subprocess.PIPE) == 0:
+                break
+        if timeout <= 0:
+            with open(log) as f:
+                sys.stderr.write('Cannot start X.org with dummy driver.  Log:\n%s\n--------' % f.read())
+            sys.exit(1)
+
+    @classmethod
+    def stop_xorg(klass):
+        '''stop X.org server with dummy driver'''
+
+        klass.xorg.terminate()
+        klass.xorg.wait()
+
     @classmethod
     def reset_idle_timer(klass):
         '''trigger activity to reset idle timer'''
